from sqlalchemy import Column, Integer, String

from petisco.persistence.persistence import Persistence

Base = Persistence.get_base("petisco-sql")


class UserModel(Base):
    __tablename__ = "User"

    id = Column("id", Integer, primary_key=True)
    user_id = Column("user_id", String(12))
    client_id = Column("client_id", String(50))
    name = Column("name", String(50))



import base64
import os
import re

from petisco.domain.value_objects.uuid import Uuid, InvalidUuidError

LENGTH = 16


class UserId(Uuid):
    @classmethod
    def generate_legacy(cls):
        return cls(LegacyUserId.generate().value, execute_guard=False)

    def __init__(self, value, execute_guard: bool = True):
        try:
            super(UserId, self).__init__(value, execute_guard=execute_guard)
        except InvalidUuidError:
            legacy_user_id = LegacyUserId(value)
            super(UserId, self).__init__(legacy_user_id.value, execute_guard=False)


class LegacyUserId(Uuid):
    def guard(self):
        self._ensure_is_16_char_uuid()
        self._ensure_value_contains_valid_char()

    def _ensure_is_16_char_uuid(self):
        if self.value is None or len(self.value) != LENGTH:
            raise InvalidUuidError(self.value)

    def _ensure_value_contains_valid_char(self, allow_utf8mb4: bool = True):
        if not isinstance(self.value, str) or not re.search(
            r"^[\w]*(([',. -][\s]?[\w]?)?[\w]*)*$", self.value
        ):
            raise InvalidUuidError(self.value)
        if not allow_utf8mb4 and re.match("[^\u0000-\uffff]", self.value):
            raise InvalidUuidError(self.value)

    @classmethod
    def generate(cls):
        # TODO: refactor to cls(str(uuid.uuid4()))
        r = os.urandom(LENGTH)
        return cls(base64.b64encode(r, altchars=b"-_").decode("utf-8")[:LENGTH])



from typing import Dict

from petisco.notifier.domain.interface_notifier import INotifier
from petisco.notifier.domain.notifier_message import NotifierMessage


class FakeNotifier(INotifier):
    def __init__(self):
        self.publish_called = False
        self.publish_times_called = 0

    def info(self) -> Dict:
        pass

    def publish(self, notifier_message: NotifierMessage):
        self.publish_called = True
        self.publish_times_called += 1



from typing import Callable, List

from sqlalchemy import text


class SqlExecutor:
    def __init__(self, session_scope: Callable):
        self.session_scope = session_scope

    def _get_command(self, statement: str):
        command = text(statement.rstrip("\n"))
        return command

    def execute_from_filename(self, filename: str):
        with open(filename) as file:
            statements = file.read().split(";")
            self.execute_statements(statements)

    def execute_statement(self, statement: str):
        with self.session_scope() as session:
            command = self._get_command(statement)
            session.execute(command)

    def execute_statements(self, statements: List[str]):
        with self.session_scope() as session:
            for statement in statements:
                command = self._get_command(statement)
                session.execute(command)



import json
from unittest.mock import patch, MagicMock

import pytest
from slack import WebClient

from petisco.notifier.domain.notifier_message import NotifierMessage
from petisco.notifier.infrastructure.slack.slack_notifier import SlackNotifier


@pytest.mark.integration
@patch.object(WebClient, "chat_postMessage")
def test_should_publish_a_notifier_message(mock_slack_client):

    notifier = SlackNotifier(token="test_token", channel="#feed")
    notifier.publish(NotifierMessage(message="Test message"))
    mock_slack_client.return_value = MagicMock(
        status_code=200, response=json.dumps({"key": "value"})
    )
    mock_slack_client.assert_called_once()



import pytest

from petisco import WebhookSenderFailed
from petisco.webhooks.webhook.domain.webhook_result import WebhookResult
from tests.modules.webook.mothers.webhook_result_mother import WebhookResultMother


@pytest.mark.unit
@pytest.mark.parametrize(
    "webhook_result",
    [
        WebhookResultMother.success(),
        WebhookResultMother.failure(),
        WebhookResultMother.without_response(),
        WebhookResultMother.without_request(),
        WebhookResultMother.without_request_headers(),
    ],
)
def test_should_serialize_and_deserialize_a_webhook_result(webhook_result):
    serialized = webhook_result.to_dict()

    deserialized_webhook_result = WebhookResult.from_dict(serialized)

    assert webhook_result == deserialized_webhook_result


@pytest.mark.unit
def test_should_check_not_events_when_success():
    webhook_result = WebhookResultMother.success_from_create()
    assert len(webhook_result.pull_domain_events()) == 0


@pytest.mark.unit
def test_should_check_webhook_sender_failed_event_when_is_failure():
    webhook_result = WebhookResultMother.failure_from_create()
    assert len(webhook_result.pull_domain_events()) == 1
    assert isinstance(webhook_result.pull_first_domain_event(), WebhookSenderFailed)



from typing import Dict

from petisco.event.legacy.subscriber.domain.interface_event_subscriber import (
    IEventSubscriber,
)


class NotImplementedEventSubscriber(IEventSubscriber):
    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def start(self):
        pass

    def stop(self):
        pass



import pytest
from flask import Response

from petisco.event.shared.domain.request_responded import RequestResponded


@pytest.fixture
def given_tuple_http_response():
    return {"message": "ok"}, 200


@pytest.fixture
def given_tuple_http_response_with_long_content():
    return {"message": "ok" * 300}, 200


@pytest.fixture
def given_tuple_http_response_with_long_content_and_406():
    return {"error": {"type": "MyError", "message": "no" * 300}}, 406


@pytest.fixture
def given_tuple_flask_http_response():
    return Response(), 200


@pytest.fixture
def given_tuple_flask_response():
    return Response()


@pytest.mark.unit
def test_should_construct_request_responded_from_a_tuple_http_response(
    given_tuple_http_response, given_any_info_id
):
    request_responded = RequestResponded(
        app_name="petisco",
        app_version="1.0.0",
        controller="my-controller",
        is_success=True,
        http_response=given_tuple_http_response,
        elapsed_time=1.0,
    ).add_info_id(given_any_info_id)

    assert request_responded.event_name == "request.responded"
    assert request_responded.http_response["content"] == {
        "message": '{"message": "ok"}',
        "message_size": 17,
    }
    assert request_responded.http_response["status_code"] == 200


@pytest.mark.unit
def test_should_construct_request_responded_from_a_tuple_http_response_with_long_content(
    given_tuple_http_response_with_long_content, given_any_info_id
):
    request_responded = RequestResponded(
        app_name="petisco",
        app_version="1.0.0",
        controller="my-controller",
        is_success=True,
        http_response=given_tuple_http_response_with_long_content,
        elapsed_time=1.0,
    ).add_info_id(given_any_info_id)

    assert request_responded.event_name == "request.responded"
    assert request_responded.http_response["content"] == {
        "message": "Response OK (Trimmed message: message too long)",
        "message_size": 615,
    }
    assert request_responded.http_response["status_code"] == 200


@pytest.mark.unit
def test_should_construct_request_responded_from_a_tuple_http_response_with_long_content_and_406(
    given_tuple_http_response_with_long_content_and_406, given_any_info_id
):
    request_responded = RequestResponded(
        app_name="petisco",
        app_version="1.0.0",
        controller="my-controller",
        is_success=True,
        http_response=given_tuple_http_response_with_long_content_and_406,
        elapsed_time=1.0,
    ).add_info_id(given_any_info_id)

    assert request_responded.event_name == "request.responded"
    assert request_responded.http_response["content"] == {
        "error": {
            "type": "MyError",
            "message": "Response Error (Trimmed message: message too long)",
        },
        "message_size": 645,
    }
    assert request_responded.http_response["status_code"] == 406


@pytest.mark.unit
def test_should_construct_request_responded_from_a_tuple_flask_http_response(
    given_tuple_flask_http_response, given_any_info_id
):
    request_responded = RequestResponded(
        app_name="petisco",
        app_version="1.0.0",
        controller="my-controller",
        is_success=True,
        http_response=given_tuple_flask_http_response,
        elapsed_time=1.0,
    ).add_info_id(given_any_info_id)

    assert request_responded.http_response["content"] == {"message": "flask response"}
    assert request_responded.http_response["status_code"] == 200


@pytest.mark.unit
def test_should_construct_request_responded_from_flask_http_response(
    given_tuple_flask_response, given_any_info_id
):
    request_responded = RequestResponded(
        app_name="petisco",
        app_version="1.0.0",
        controller="my-controller",
        is_success=True,
        http_response=given_tuple_flask_response,
        elapsed_time=1.0,
    ).add_info_id(given_any_info_id)

    assert request_responded.http_response["content"] == {"message": "flask response"}
    assert request_responded.http_response["status_code"] == 200



from typing import Dict

from petisco.notifier.domain.interface_notifier import INotifier
from petisco.notifier.domain.notifier_message import NotifierMessage


class NotImplementedNotifier(INotifier):
    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def publish(self, message: NotifierMessage):
        pass



import os


class ElasticConnection:
    def __init__(
        self,
        username: str = None,
        password: str = None,
        host: str = "localhost",
        port: str = "9200",
    ):
        self.username = username
        self.password = password
        self.host = host
        self.port = port

    @property
    def http_auth(self):
        http_auth = None
        if self.username and self.password:
            http_auth = (self.username, self.password)
        return http_auth

    @staticmethod
    def create(username: str, password: str, host: str, port: str):
        return ElasticConnection(username, password, host, port)

    @staticmethod
    def create_local():
        return ElasticConnection(host="localhost")

    @staticmethod
    def from_environ():
        return ElasticConnection.create(
            os.getenv("ELASTIC_USERNAME", ""),
            os.getenv("ELASTIC_PASSWORD", ""),
            os.getenv("ELASTIC_HOST", "localhost"),
            os.getenv("ELASTIC_PORT", "9200"),
        )



import pytest

from petisco import Webhook
from petisco.webhooks.webhook.domain.invalid_url_error import InvalidUrlError
from petisco.webhooks.webhook.domain.webhook_created import WebhookCreated
from tests.modules.webook.mothers.webhook_mother import WebhookMother


@pytest.mark.unit
def test_should_construct_a_valid_webhook():
    webhook = WebhookMother.default()
    assert isinstance(webhook, Webhook)
    assert isinstance(webhook.pull_domain_events()[0], WebhookCreated)


@pytest.mark.unit
def test_should_raise_an_error_when_construct_a_webhook_with_invalid_url():

    with pytest.raises(InvalidUrlError):
        WebhookMother.with_invalid_url()


@pytest.mark.unit
def test_should_serialize_successfully():
    original_webhook = WebhookMother.default()
    webhook_data = original_webhook.to_dict()
    retrieved_webhook = Webhook.from_dict(webhook_data)
    assert original_webhook == retrieved_webhook


@pytest.mark.unit
def test_should_check_active_attribute_on_webhooks():
    assert WebhookMother.default().active
    assert not WebhookMother.not_active().active



from meiga import Result

from petisco import (
    controller_handler,
    TokenManager,
    HttpError,
    InfoId,
    UserId,
    AcceptedToken,
)
from petisco.domain.value_objects.name import Name
from petisco.domain.errors.given_input_is_not_valid_error import (
    GivenInputIsNotValidError,
)
from petisco.domain.errors.given_name_is_not_valid_error import GivenNameIsNotValidError
from tests.end2end.flask_app.user.application.user_creator import UserCreator


def success_handler(result: Result):
    return {"user_id": result.value.value}, 200


class GivenInputIsNotValidHttpError(HttpError):
    def __init__(self, message: str = "Given input is not valid", code: int = 409):
        super(GivenInputIsNotValidHttpError, self).__init__(message=message, code=code)


def error_handler(result: Result):
    print(f"error_handler: {result}")
    domain_error = result.value
    if isinstance(domain_error, (GivenNameIsNotValidError, GivenInputIsNotValidError)):
        return GivenInputIsNotValidHttpError()


@controller_handler(
    success_handler=success_handler,
    error_handler=error_handler,
    token_manager=TokenManager(
        accepted_tokens=[AcceptedToken(token_type="ADMIN_TOKEN")]
    ),
    send_request_responded_event=True,
)
def create_user(info_id: InfoId, body: dict):
    info_id.user_id = UserId.generate()
    name = Name(body.get("name"))
    return UserCreator.build().execute(info_id=info_id, name=name)



from meiga import Failure, isSuccess, BoolResult

from petisco.security.token_decoder.invalid_token_error import InvalidTokenError
from petisco.security.token_decoder.token import Token


class AcceptedToken:
    def __init__(self, token_type: str, require_user: bool = False):
        self.token_type = token_type
        self.require_user = require_user

    @staticmethod
    def user():
        return AcceptedToken(token_type="USER", require_user=True)

    @staticmethod
    def backend():
        return AcceptedToken(token_type="BACKEND")

    @staticmethod
    def backend_with_user():
        return AcceptedToken(token_type="BACKEND", require_user=True)

    def check(self, token: Token) -> BoolResult:
        if (
            token.token_type != self.token_type
            or (self.require_user and not token.user_id)
            or (not self.require_user and token.user_id)
        ):
            return Failure(
                InvalidTokenError(
                    message=f"This entry point expects a valid {self.token_type} Token with require_user={self.require_user}"
                )
            )
        return isSuccess



import pytest

from petisco.event.shared.domain.config_events import ConfigEvents


@pytest.fixture
def given_valid_event_config():
    return {
        "events": {
            "message_broker": "not_implemented",
            "organization": "alice",
            "service": "petisco",
            "subscribers": {
                "UserCreated": {
                    "version": 1,
                    "handlers": "tests.modules.event.unit.callable_functions.handlers.send_mail_handler",
                },
                "UserUpdated": {
                    "version": 1,
                    "handlers": [
                        "tests.modules.event.unit.callable_functions.handlers.send_mail_handler",
                        "tests.modules.event.unit.callable_functions.handlers.send_sms_handler",
                    ],
                },
            },
        }
    }


@pytest.fixture
def given_invalid_handlers_on_event_config():
    return {
        "events": {
            "message_broker": "not_implemented",
            "organization": "alice",
            "service": "petisco",
            "subscribers": {
                "UserCreated": {
                    "version": 1,
                    "handlers": "not.valid.handlers.send_mail_handler",
                },
                "UserUpdated": {
                    "version": 1,
                    "handlers": [
                        "tests.modules.event.unit.callable_functions.handlers.send_mail_handler",
                        "tests.modules.event.unit.callable_functions.handlers.send_sms_handler",
                    ],
                },
            },
        }
    }


@pytest.fixture
def given_valid_event_config_with_queue_subscribers():
    return {
        "events": {
            "message_broker": "not_implemented",
            "organization": "alice",
            "service": "petisco",
            "subscribers": {
                "UserCreated": {
                    "version": 1,
                    "handlers": "tests.modules.event.unit.callable_functions.handlers.send_mail_handler",
                },
                "UserUpdated": {
                    "version": 1,
                    "handlers": [
                        "tests.modules.event.unit.callable_functions.handlers.send_mail_handler",
                        "tests.modules.event.unit.callable_functions.handlers.send_sms_handler",
                    ],
                },
            },
        },
        "queues": {
            "store": "tests.modules.event.unit.callable_functions.handlers.event_store",
            "subscribers": {
                "dead_letter.store": "tests.modules.event.unit.callable_functions.handlers.requeue_event"
            },
        },
    }


@pytest.mark.unit
def test_should_load_config_event_from_valid_dictionary(given_valid_event_config):
    config_events = ConfigEvents.from_dict(given_valid_event_config)

    assert isinstance(config_events, ConfigEvents)


@pytest.mark.unit
def test_should_load_config_event_from_valid_dictionary_with_queue_subscribers(
    given_valid_event_config_with_queue_subscribers
):
    config_events = ConfigEvents.from_dict(
        given_valid_event_config_with_queue_subscribers
    )

    assert isinstance(config_events, ConfigEvents)


@pytest.mark.unit
def test_should_raise_an_error_given_invalid_handlers_on_event_config(
    given_invalid_handlers_on_event_config
):

    with pytest.raises(TypeError):
        _ = ConfigEvents.from_dict(given_invalid_handlers_on_event_config)



from typing import Dict

from meiga import Result, Error, NotImplementedMethodError

from petisco.application.interface_repository import IRepository


class IUsersCountRepository(IRepository):
    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def increase(self) -> Result[bool, Error]:
        return NotImplementedMethodError

    def decrease(self) -> Result[bool, Error]:
        return NotImplementedMethodError

    def count(self) -> Result[int, Error]:
        return NotImplementedMethodError



import pytest


@pytest.fixture
def given_any_token_type():
    return "TOKEN"


@pytest.fixture
def given_other_token_type():
    return "REQUIRED_TOKEN"


@pytest.fixture
def given_any_token_type_with_user():
    return "USER_TOKEN"


@pytest.fixture
def given_any_decoded_token_info(given_any_token_type, given_any_client_id):
    return {
        "user_id": None,
        "client_id": given_any_client_id,
        "token_type": given_any_token_type,
    }


@pytest.fixture
def given_any_decoded_token_info_with_user(
    given_any_token_type_with_user, given_any_client_id, given_any_user_id
):
    return {
        "user_id": given_any_user_id,
        "client_id": given_any_client_id,
        "token_type": given_any_token_type_with_user,
    }



from meiga import Result, Success, Error

from petisco import UseCase, use_case_handler, InfoId, Petisco, Repositories
from petisco.domain.value_objects.name import Name
from petisco.domain.value_objects.user_id import UserId
from petisco.event.bus.domain.interface_event_bus import IEventBus
from tests.end2end.flask_app.shared.domain.repositories.interface_user_repository import (
    IUserRepository,
)
from tests.end2end.flask_app.user.domain.aggregate_roots.user import User


@use_case_handler()
class UserCreator(UseCase):
    @staticmethod
    def build():
        return UserCreator(
            repository=Repositories.get("user"), bus=Petisco.get_event_bus()
        )

    def __init__(self, repository: IUserRepository, bus: IEventBus):
        self.repository = repository
        self.bus = bus

    def execute(self, info_id: InfoId, name: Name) -> Result[UserId, Error]:
        user = User.create(info_id, name)
        self.repository.save(user).unwrap_or_return()
        self.bus.publish_events(user.pull_domain_events())
        return Success(user.user_id)



import os
import re
import traceback
from typing import Optional, List, Any, Callable, Dict

import yaml
from dataclasses import dataclass
from meiga import Result, Error, Failure, Success
from meiga.decorators import meiga

from petisco.event.chaos.infrastructure.rabbitmq_event_chaos import RabbitMqEventChaos
from petisco.event.chaos.domain.interface_event_chaos import IEventChaos
from petisco.application.config.check_list_or_str_item import check_list_or_str_item
from petisco.application.config.get_funtion_from_string import get_function_from_string
from petisco.application.config.raise_petisco_config_error import (
    raise_petisco_config_exception,
)
from petisco.event.queue.domain.queue_config import QueueConfig

from petisco.event.shared.domain.event_subscriber import EventSubscriber
from petisco.application.config.config_file_not_found_error import (
    ConfigFileNotFoundError,
)
from yaml.parser import ParserError, ScannerError

from petisco.application.config.config_file_not_valid_error import (
    ConfigFileNotValidError,
)


def get_event_name(src_event_name: str) -> str:
    event_name = (
        re.sub(r"(?<!^)(?=[A-Z])", "_", src_event_name).lower().replace(".", "_")
    )
    return event_name


def get_handlers(handlers_names: List[str], kdict: Dict) -> List[Callable]:
    handlers = []
    for handler_name in handlers_names:
        handler = (
            get_function_from_string(handler_name)
            .handle(on_failure=raise_petisco_config_exception, failure_args=kdict)
            .unwrap()
        )
        handlers.append(handler)
    return handlers


DEFAULT_QUEUE_RETRY_TTL = 5000
DEFAULT_QUEUE_MAIN_TTL = 5000


@dataclass
class ConfigEvents:
    organization: str
    service: str
    publish_deploy_event: bool = False
    consumer_verbose: bool = False
    use_store_queues: bool = True
    retry_ttl: int = 5000
    main_ttl: int = 5000
    max_retries: int = 5
    message_broker: Optional[str] = "not_implemented"
    event_subscribers: Optional[List[EventSubscriber]] = None
    store_queue_subscriber: Optional[Callable] = None
    queues_subscribers: Optional[Dict[str, Callable]] = None
    chaos: Optional[IEventChaos] = None
    queue_config: Optional[QueueConfig] = None

    def __post_init__(self):
        message_broker = os.environ.get("PETISCO_EVENT_MESSAGE_BROKER")
        if message_broker:
            self.message_broker = message_broker

    @staticmethod
    @meiga
    def from_filename(filename: str) -> Result[Any, Error]:
        if not os.path.isfile(filename):
            return Failure(ConfigFileNotFoundError(filename))
        try:
            with open(filename) as file:
                petisco_dict = yaml.load(file, Loader=yaml.FullLoader)
                config_event = ConfigEvents.from_dict(petisco_dict)
                return Success(config_event)
        except (ParserError, ScannerError) as e:
            message = f"Error loading {filename} file: {repr(e.__class__)} {e} | {traceback.format_exc()}"
            return Failure(ConfigFileNotValidError(message))

    @staticmethod
    def from_dict(kdict):
        event_subscribers = None
        queues_subscribers = None
        store_queue_subscriber = None

        events = kdict.get("events")
        queues = kdict.get("queues")

        retry_ttl = DEFAULT_QUEUE_RETRY_TTL
        main_ttl = DEFAULT_QUEUE_RETRY_TTL
        queue_config = QueueConfig.default(retry_ttl, main_ttl)

        if events:
            retry_ttl = events.get("retry_ttl", DEFAULT_QUEUE_RETRY_TTL)
            main_ttl = events.get("retry_ttl", DEFAULT_QUEUE_RETRY_TTL)
            queue_config = QueueConfig.default(retry_ttl, main_ttl)

            dict_events_subscribers = events.get("subscribers")
            if dict_events_subscribers:
                event_subscribers = []
                for src_event_name, event_info in dict_events_subscribers.items():
                    handlers_names = check_list_or_str_item(
                        event_info, "handlers", typename="EventHandler"
                    )

                    event_name = get_event_name(src_event_name)
                    event_version = event_info.get("version", 1)
                    handlers = get_handlers(handlers_names, kdict)

                    event_subscribers.append(
                        EventSubscriber(
                            event_name=event_name,
                            event_version=event_version,
                            handlers=handlers,
                        )
                    )

        if queues:
            str_store_queue_subscriber = queues.get("store")
            if str_store_queue_subscriber:
                store_queue_subscriber = get_handlers(
                    [str_store_queue_subscriber], kdict
                )[0]

            dict_queues_subscribers = queues.get("subscribers")
            if dict_queues_subscribers:
                queues_subscribers = {}
                for queue_name in dict_queues_subscribers.keys():
                    handlers_names = check_list_or_str_item(
                        dict_queues_subscribers, queue_name, typename="QueueHandler"
                    )
                    handlers = get_handlers(handlers_names, kdict)
                    queues_subscribers[queue_name] = handlers

            specific_queues_config = queues.get("specific_config")
            if specific_queues_config:
                queue_config = QueueConfig.from_dict(
                    specific_queues_config, retry_ttl, main_ttl
                )

        return ConfigEvents(
            organization=events.get("organization"),
            service=events.get("service"),
            publish_deploy_event=events.get("publish_deploy_event"),
            consumer_verbose=events.get("consumer_verbose", False),
            use_store_queues=events.get("use_store_queues", True),
            retry_ttl=retry_ttl,
            main_ttl=main_ttl,
            max_retries=events.get("max_retries", 5),
            message_broker=events.get("message_broker", "not_implemented"),
            event_subscribers=event_subscribers,
            store_queue_subscriber=store_queue_subscriber,
            queues_subscribers=queues_subscribers,
            chaos=RabbitMqEventChaos(),
            queue_config=queue_config,
        )

    def info(self):
        return {
            "organization": self.organization,
            "service": self.service,
            "publish_deploy_event": self.publish_deploy_event,
            "consumer_verbose": self.consumer_verbose,
            "use_store_queues": self.use_store_queues,
            "retry_ttl": self.retry_ttl,
            "main_ttl": self.main_ttl,
            "max_retries": self.max_retries,
            "event_subscribers": self.event_subscribers,
            "store_queue_subscriber": self.store_queue_subscriber,
            "queues_subscribers": self.queues_subscribers,
            "chaos": self.chaos.info(),
            "queues_specific_config": self.queue_config.info(),
        }
